@@ -1,8 +1,8 @@
 /*
-$Id: dilib.c,v 1.37 2008-05-25 20:33:06-07 bll Exp $
-$Source: /export/home/bll/DI/RCS/dilib.c,v $
-Copyright 1994-2008 Brad Lanam, Walnut Creek, CA
-*/
+ * $Id: dilib.c,v 1.50 2009-11-22 15:53:11-08 bll Exp $
+ * $Source: /home/bll/DI/RCS/dilib.c,v $
+ * Copyright 1994-2009 Brad Lanam, Walnut Creek, CA
+ */
 
 /********************************************************/
 /*
@@ -15,17 +15,24 @@ Copyright 1994-2008 Brad Lanam, Walnut Creek, CA
     di_testRemoteDisk ()
         Checks the partitions to see if they are mounted
         locally or not.
-    di_getDiskInfo
+    di_getDiskInfo ()
         Gets the disk space used/available on the
         partitions we want displayed.
-
-    To Do:
-        The following operating systems have
-        empty di_getDiskInfo() routines:
-            BeOS OSF/1 Ultrix
-        These need to be rewritten to separate
-        the fetch of mounted partition info, and
-        the fetch of disk space.
+    checkMountOptions ()
+        Checks to see if the mount option is set.
+    chkMountOptions ()
+        Checks to see if the mount option is set.
+        Used if hasmntopt() is not present.
+    Realloc ()
+        Portable realloc.
+    convertMountOptions ()
+        converts mount options to text format.
+    convertNFSMountOptions ()
+        converts NFS mount options to text format.
+    trimChar ()
+        removes a character from a string.
+    di_testRemoteDisk ()
+        test a disk to see if it is remote (nfs, nfs3).
 
 */
 /********************************************************/
@@ -138,65 +145,128 @@ Copyright 1994-2008 Brad Lanam, Walnut Creek, CA
 #endif
 #if _hdr_storage_Path
 # include <storage/Path.h>
+#endif
+ /* bozo syllable volume header requires gui/window */
+#if _hdr_gui_window
+# include <gui/window.h>
+#endif
+#if _hdr_storage_volumes
+# include <storage/volumes.h>
+#endif
+#if _hdr_util_string
+# include <util/string.h>
 #endif
 
 /********************************************************/
 
     /* remap mount flags */
-# if defined (M_RDONLY)
-#  define MNT_RDONLY M_RDONLY
-# endif
-# if defined (MNT_READONLY)
-#  define MNT_RDONLY MNT_READONLY
-# endif
-# if defined (M_RONLY)
-#  define MNT_RDONLY M_RONLY
-# endif
-# if defined (M_SYNCHRONOUS)
-#  define MNT_SYNCHRONOUS M_SYNCHRONOUS
-# endif
-# if defined (M_NOEXEC)
-#  define MNT_NOEXEC M_NOEXEC
-# endif
-# if defined (M_NOSUID)
-#  define MNT_NOSUID M_NOSUID
-# endif
-# if defined (M_NODEV)
-#  define MNT_NODEV M_NODEV
-# endif
-# if defined (M_NOATIMES)
-#  define MNT_NOATIMES M_NOATIMES
-# endif
-# if defined (M_GRPID)
-#  define MNT_GRPID M_GRPID
-# endif
-# if defined (M_SECURE)
-#  define MNT_SECURE M_SECURE
-# endif
-# if defined (M_MLSD)
-#  define MNT_MLSD M_MLSD
-# endif
-# if defined (M_SMSYNC2)
-#  define MNT_SMSYNC2 M_SMSYNC2
-# endif
-# if defined (M_LOCAL)
-#  define MNT_LOCAL M_LOCAL
-# endif
-# if defined (M_FORCE)
-#  define MNT_FORCE M_FORCE
-# endif
-# if defined (M_SYNC)
-#  define MNT_SYNC M_SYNC
-# endif
-# if defined (M_NOCACHE)
-#  define MNT_NOCACHE M_NOCACHE
-# endif
+#if defined (B_FS_IS_READONLY)
+# define MNT_RDONLY B_FS_IS_READONLY
+#endif
+#if defined (FS_IS_READONLY)
+# define MNT_RDONLY FS_IS_READONLY
+#endif
+#if defined (M_RDONLY)
+# define MNT_RDONLY M_RDONLY
+#endif
+#if defined (MNT_READONLY)
+# define MNT_RDONLY MNT_READONLY
+#endif
+#if defined (M_RONLY)
+# define MNT_RDONLY M_RONLY
+#endif
+#if defined (M_SYNCHRONOUS)
+# define MNT_SYNCHRONOUS M_SYNCHRONOUS
+#endif
+#if defined (M_NOEXEC)
+# define MNT_NOEXEC M_NOEXEC
+#endif
+#if defined (M_NOSUID)
+# define MNT_NOSUID M_NOSUID
+#endif
+#if defined (M_NODEV)
+# define MNT_NODEV M_NODEV
+#endif
+#if defined (M_NOATIMES)
+# define MNT_NOATIMES M_NOATIMES
+#endif
+#if defined (M_GRPID)
+# define MNT_GRPID M_GRPID
+#endif
+#if defined (M_SECURE)
+# define MNT_SECURE M_SECURE
+#endif
+#if defined (M_MLSD)
+# define MNT_MLSD M_MLSD
+#endif
+#if defined (M_SMSYNC2)
+# define MNT_SMSYNC2 M_SMSYNC2
+#endif
+#if defined (M_LOCAL)
+# define MNT_LOCAL M_LOCAL
+#endif
+#if defined (M_FORCE)
+# define MNT_FORCE M_FORCE
+#endif
+#if defined (M_SYNC)
+# define MNT_SYNC M_SYNC
+#endif
+#if defined (M_NOCACHE)
+# define MNT_NOCACHE M_NOCACHE
+#endif
+#if defined (B_FS_IS_REMOVABLE)
+# define MNT_REMOVABLE B_FS_IS_REMOVABLE
+#endif
+#if defined (FS_IS_REMOVABLE)
+# define MNT_REMOVABLE FS_IS_REMOVABLE
+#endif
+#if defined (B_FS_IS_PERSISTENT)
+# define MNT_PERSISTENT B_FS_IS_PERSISTENT
+#endif
+#if defined (FS_IS_PERSISTENT)
+# define MNT_PERSISTENT FS_IS_PERSISTENT
+#endif
+#if defined (B_FS_IS_SHARED)
+# define MNT_SHARED B_FS_IS_SHARED
+#endif
+#if defined (FS_IS_SHARED)
+# define MNT_SHARED FS_IS_SHARED
+#endif
+#if defined (FS_IS_BLOCKBASED)
+# define MNT_BLOCKBASED FS_IS_BLOCKBASED
+#endif
+#if defined (B_FS_HAS_MIME)
+# define MNT_HAS_MIME B_FS_HAS_MIME
+#endif
+#if defined (FS_HAS_MIME)
+# define MNT_HAS_MIME FS_HAS_MIME
+#endif
+#if defined (B_FS_HAS_ATTR)
+# define MNT_HAS_ATTR B_FS_HAS_ATTR
+#endif
+#if defined (FS_HAS_ATTR)
+# define MNT_HAS_ATTR FS_HAS_ATTR
+#endif
+#if defined (B_FS_HAS_QUERY)
+# define MNT_HAS_QUERY B_FS_HAS_QUERY
+#endif
+#if defined (FS_HAS_QUERY)
+# define MNT_HAS_QUERY FS_HAS_QUERY
+#endif
+
+#if defined(__cplusplus)
+  extern "C" {
+#endif
 
 static void convertMountOptions _((long, diDiskInfo_t *));
 static void convertNFSMountOptions _((long, long, long, diDiskInfo_t *));
 static void trimChar _((char *, int));
 #if _lib_getmntent && \
-	! defined (_lib_mntctl)
+    ! defined (_lib_getmntinfo) && \
+    ! defined (_lib_getfsstat) && \
+    ! defined (_lib_getvfsstat) && \
+	! defined (_lib_mntctl) && \
+	! defined (_class_os__Volumes)
 static char *chkMountOptions _((char *, char *));
 #endif
 
@@ -204,24 +274,29 @@ static char *chkMountOptions _((char *, char *));
 /********************************************************/
 
 #if (_lib_getmntent || \
-        _statfs_2arg || \
-        _statfs_3arg || \
-        _statfs_4arg) && \
-        ! defined (_lib_getmntinfo) && \
-        ! defined (_lib_getfsstat) && \
-        ! defined (_lib_getvfsstat) && \
-        ! defined (_lib_mntctl) && \
-        ! defined (_lib_getmnt)
-# if defined (MOUNTED)
-#  define DI_MOUNT_FILE        MOUNTED
+    _statfs_2arg || \
+    _statfs_3arg || \
+    _statfs_4arg) && \
+    ! defined (_lib_getmntinfo) && \
+    ! defined (_lib_getfsstat) && \
+    ! defined (_lib_getvfsstat) && \
+    ! defined (_lib_mntctl) && \
+    ! defined (_lib_getmnt) && \
+    ! defined (_class_os__Volumes)
+# if defined (_PATH_MOUNTED)
+#  define DI_MOUNT_FILE        _PATH_MOUNTED
 # else
-#  if defined (MNTTAB)
-#   define DI_MOUNT_FILE       MNTTAB
+#  if defined (MOUNTED)
+#   define DI_MOUNT_FILE       MOUNTED
 #  else
-#   if (USE_ETC_FILESYSTEMS)
-#    define DI_MOUNT_FILE       "/etc/filesystems" /* AIX 4.x or /etc/mntent? */
+#   if defined (MNTTAB)
+#    define DI_MOUNT_FILE      MNTTAB
 #   else
-#    define DI_MOUNT_FILE       "/etc/mnttab"
+#    if (USE_ETC_FILESYSTEMS)
+#     define DI_MOUNT_FILE     "/etc/filesystems" /* AIX 4.x or /etc/mntent? */
+#    else
+#     define DI_MOUNT_FILE     "/etc/mnttab"
+#    endif
 #   endif
 #  endif
 # endif
@@ -230,18 +305,22 @@ static char *chkMountOptions _((char *, char *));
 #if ! defined (_lib_statvfs) && \
 	_lib_statfs && \
 	_npt_statfs
-# if _statfs_2arg
+# if _lib_statfs && _statfs_2arg
   extern int statfs _((char *, struct statfs *));
 # endif
-# if _statfs_3arg
+# if _lib_statfs && _statfs_3arg
   extern int statfs _((char *, struct statfs *, int));
 # endif
-# if _statfs_4arg
+# if _lib_statfs && _statfs_4arg
   extern int statfs _((char *, struct statfs *, int, int));
 # endif
 #endif
 
-#if _statfs_4arg && \
+#if defined(__cplusplus)
+  }
+#endif
+
+#if _lib_statfs && _statfs_4arg && \
     ! defined (_lib_statvfs) && \
     ! defined (_lib_getmntinfo) && \
     ! defined (_lib_getfsstat) && \
@@ -278,11 +357,12 @@ static char *chkMountOptions _((char *, char *));
 
 /********************************************************/
 
-#define DI_UNKNOWN_FSTYPE       "Unknown fstyp %.2d"
+#define DI_UNKNOWN_FSTYPE       "Unknown fstype %.2d"
 
 int di_lib_debug = { 0 };
 
-#if _lib_fs_stat_dev
+#if _lib_fs_stat_dev && \
+    _lib_next_dev
 
 /*
  * di_getDiskEntries
@@ -334,7 +414,7 @@ di_getDiskEntries (diskInfo, diCount)
         dir = new BDirectory (&nref);
         stat = dir->GetEntry (&entry);
         stat = entry.GetPath (&path);
-        strncpy (diptr->name, path.Path (), DI_NAME_LEN);
+        strncpy (diptr->name, path.Path(), DI_NAME_LEN);
         strncpy (diptr->special, fsinfo.device_name, DI_SPEC_NAME_LEN);
         strncpy (diptr->fsType, fsinfo.fsh_name, DI_TYPE_LEN);
         diptr->isLocal = TRUE;
@@ -346,11 +426,24 @@ di_getDiskEntries (diskInfo, diCount)
         diptr->totalInodes = fsinfo.total_nodes;
         diptr->freeInodes = fsinfo.free_nodes;
         diptr->availInodes = fsinfo.free_nodes;
+# if defined (MNT_RDONLY)
+        if ((fsinfo.flags & MNT_RDONLY) == MNT_RDONLY)
+        {
+           diptr->isReadOnly = TRUE;
+        }
+# endif
+# if defined (MNT_PERSISTENT)
+        if ((fsinfo.flags & MNT_PERSISTENT) != MNT_PERSISTENT)
+        {
+           diptr->printFlag = DI_PRNT_IGNORE;
+        }
+# endif
+        convertMountOptions ((long) fsinfo.flags, diptr);
+        trimChar (diptr->options, ',');
 
         if (di_lib_debug > 0)
         {
-            printf ("mnt:%s - %s\n", diptr->name,
-                    diptr->special);
+            printf ("mnt:%s - %s\n", diptr->name, diptr->special);
             printf ("dev:%d fs:%s\n", dev, diptr->fsType);
         }
         if (di_lib_debug > 1)
@@ -367,9 +460,103 @@ di_getDiskEntries (diskInfo, diCount)
 
 #endif
 
+#if _class_os__Volumes
+
+/*
+ * di_getDiskEntries
+ *
+ * For Syllable.
+ *
+ */
+
+int
+# if _proto_stdc
+di_getDiskEntries (diDiskInfo_t **diskInfo, int *diCount)
+# else
+di_getDiskEntries (diskInfo, diCount)
+    diDiskInfo_t **diskInfo;
+    int *diCount;
+# endif
+{
+    diDiskInfo_t    *diptr;
+    int             idx;
+    Uint_t          i;
+    status_t        stat;
+    uint32          count;
+    fs_info         fsinfo;
+    os::Volumes     *vols;
+    os::String      mpString;
+
+    if (di_lib_debug > 0) { printf ("# lib:getDiskEntries: os::Volumes\n"); }
+    vols = new os::Volumes;
+    count = vols->GetMountPointCount ();
+    for (i = 0; i < count; ++i)
+    {
+        if ((stat = vols->GetFSInfo (i, &fsinfo)) != 0)
+        {
+           break;
+        }
+
+        idx = *diCount;
+        ++*diCount;
+        *diskInfo = (diDiskInfo_t *) Realloc ((char *) *diskInfo,
+                sizeof (diDiskInfo_t) * *diCount);
+        diptr = *diskInfo + idx;
+        memset ((char *) diptr, '\0', sizeof (diDiskInfo_t));
+        diptr->printFlag = DI_PRNT_OK;
+        stat = vols->GetMountPoint (i, &mpString);
+        strncpy (diptr->name, mpString.c_str(), DI_NAME_LEN);
+        strncpy (diptr->special, fsinfo.fi_device_path, DI_SPEC_NAME_LEN);
+        strncpy (diptr->fsType, fsinfo.fi_driver_name, DI_TYPE_LEN);
+        diptr->isLocal = TRUE;
+        diptr->isReadOnly = FALSE;
+        diptr->blockSize = (_fs_size_t) fsinfo.fi_block_size;
+        diptr->totalBlocks = (_fs_size_t) fsinfo.fi_total_blocks;
+        diptr->freeBlocks = (_fs_size_t) fsinfo.fi_free_blocks;
+        diptr->availBlocks = (_fs_size_t) fsinfo.fi_free_blocks;
+        diptr->totalInodes = fsinfo.fi_total_inodes;
+        diptr->freeInodes = fsinfo.fi_free_inodes;
+        diptr->availInodes = fsinfo.fi_free_inodes;
+# if defined (MNT_RDONLY)
+        if ((fsinfo.fi_flags & MNT_RDONLY) == MNT_RDONLY)
+        {
+           diptr->isReadOnly = TRUE;
+        }
+# endif
+# if defined (MNT_PERSISTENT)
+        if ((fsinfo.fi_flags & MNT_PERSISTENT) != MNT_PERSISTENT)
+        {
+           diptr->printFlag = DI_PRNT_IGNORE;
+        }
+# endif
+        convertMountOptions ((long) fsinfo.fi_flags, diptr);
+        trimChar (diptr->options, ',');
+
+        if (di_lib_debug > 0)
+        {
+            printf ("mnt:%s - %s\n", diptr->name, diptr->special);
+            printf ("dev:%d fs:%s\n", fsinfo.fi_dev, diptr->fsType);
+        }
+        if (di_lib_debug > 1)
+        {
+            printf ("mount_args: %s\n", fsinfo.fi_mount_args);
+            printf ("%s: %s\n", diptr->name, diptr->fsType);
+            printf ("\tblocks: tot:%ld free:%ld\n",
+                    fsinfo.fi_total_blocks, fsinfo.fi_free_blocks);
+            printf ("\tinodes: tot:%ld free:%ld\n",
+                    fsinfo.fi_total_inodes, fsinfo.fi_free_inodes);
+        }
+    }
+    return 0;
+}
+
+#endif
+
+
 #if _lib_getmntent && \
 	! defined (_lib_setmntent) && \
-	! defined (_lib_mntctl)
+	! defined (_lib_mntctl) && \
+	! defined (_class_os__Volumes)
 
 /*
  * di_getDiskEntries
@@ -479,7 +666,11 @@ checkMountOptions (mntEntry, str)
 #endif /* _lib_getmntent */
 
 #if _lib_getmntent && \
-	! defined (_lib_mntctl)
+    ! defined (_lib_getmntinfo) && \
+    ! defined (_lib_getfsstat) && \
+    ! defined (_lib_getvfsstat) && \
+	! defined (_lib_mntctl) && \
+	! defined (_class_os__Volumes)
 
 static char *
 # if _proto_stdc
@@ -511,10 +702,12 @@ chkMountOptions (mntopts, str)
 #endif /* _lib_getmntent */
 
 #if _lib_statvfs && \
+    ! defined (_lib_fs_stat_dev) && \
     ! defined (_lib_getmntinfo) && \
     ! defined (_lib_getfsstat) && \
     ! defined (_lib_getvfsstat) && \
-    ! defined (_lib_GetVolumeInformation)
+    ! defined (_lib_GetVolumeInformation) && \
+	! defined (_class_os__Volumes)
 
 /*
  * di_getDiskInfo
@@ -543,7 +736,8 @@ di_getDiskInfo (diskInfo, diCount)
     {
         diptr = *diskInfo + i;
 
-        if (diptr->printFlag == DI_PRNT_OK)
+        if (diptr->printFlag == DI_PRNT_OK ||
+            diptr->printFlag == DI_PRNT_FORCE)
         {
             if (statvfs (diptr->name, &statBuf) == 0)
             {
@@ -576,17 +770,17 @@ di_getDiskInfo (diskInfo, diCount)
                     printf ("%s: %s\n", diptr->name, diptr->fsType);
                     printf ("\tbsize:%ld  frsize:%ld\n", (long) statBuf.f_bsize,
                             (long) statBuf.f_frsize);
-#if _siz_long_long >= 8
+# if _siz_long_long >= 8
                     printf ("\tblocks: tot:%llu free:%lld avail:%llu\n",
                            statBuf.f_blocks, statBuf.f_bfree, statBuf.f_bavail);
                     printf ("\tinodes: tot:%llu free:%llu avail:%llu\n",
                             statBuf.f_files, statBuf.f_ffree, statBuf.f_favail);
-#else
+# else
                     printf ("\tblocks: tot:%lu free:%lu avail:%lu\n",
                            statBuf.f_blocks, statBuf.f_bfree, statBuf.f_bavail);
                     printf ("\tinodes: tot:%lu free:%lu avail:%lu\n",
                             statBuf.f_files, statBuf.f_ffree, statBuf.f_favail);
-#endif
+# endif
                 }
             }
             else
@@ -681,7 +875,7 @@ di_getDiskEntries (diskInfo, diCount)
 
 #endif /* Sys V.3 */
 
-#if _statfs_4arg && \
+#if _lib_statfs && _statfs_4arg && \
     ! defined (_lib_statvfs) && \
     ! defined (_lib_getmntinfo) && \
     ! defined (_lib_getfsstat) && \
@@ -715,7 +909,8 @@ di_getDiskInfo (diskInfo, diCount)
     for (i = 0; i < *diCount; ++i)
     {
         diptr = *diskInfo + i;
-        if (diptr->printFlag == DI_PRNT_OK)
+        if (diptr->printFlag == DI_PRNT_OK ||
+            diptr->printFlag == DI_PRNT_FORCE)
         {
             if (statfs (diptr->name, &statBuf, sizeof (statBuf), 0) == 0)
             {
@@ -772,9 +967,13 @@ di_getDiskInfo (diskInfo, diCount)
 #if _lib_getmntent && \
     _lib_setmntent && \
     _lib_endmntent && \
+    ! defined (_lib_getmntinfo) && \
+    ! defined (_lib_getfsstat) && \
+    ! defined (_lib_getvfsstat) && \
     ! defined (_lib_mntctl) && \
     ! defined (_lib_GetDiskFreeSpace) && \
-    ! defined (_lib_GetDiskFreeSpaceEx)
+    ! defined (_lib_GetDiskFreeSpaceEx) && \
+    ! defined (_class_os__Volumes)
 
 /*
  * di_getDiskEntries
@@ -803,8 +1002,9 @@ di_getDiskEntries (diskInfo, diCount)
     char            *devp;   /* local ptr to dev entry */
 
 
-    if (di_lib_debug > 0) { printf ("# lib:getDiskEntries: get/set/endmntent\n"); }
-#if _setmntent_1arg || ! defined (_setmntent_2arg)
+    if (di_lib_debug > 0) { printf ("# lib:getDiskEntries: set/get/endmntent\n"); }
+/* if both are set not an ansi compiler... */
+#if _setmntent_1arg && ! _setmntent_2arg
     if ((f = setmntent (DI_MOUNT_FILE)) == (FILE *) NULL)
 #else
     if ((f = setmntent (DI_MOUNT_FILE, "r")) == (FILE *) NULL)
@@ -819,7 +1019,7 @@ di_getDiskEntries (diskInfo, diCount)
         idx = *diCount;
         ++*diCount;
         *diskInfo = (diDiskInfo_t *) Realloc ((char *) *diskInfo,
-                sizeof (diDiskInfo_t) * *diCount);
+                sizeof (diDiskInfo_t) * (Size_t) *diCount);
         diptr = *diskInfo + idx;
         memset ((char *) diptr, '\0', sizeof (diDiskInfo_t));
         diptr->printFlag = DI_PRNT_OK;
@@ -884,10 +1084,10 @@ di_getDiskEntries (diskInfo, diCount)
 /*
  * di_getDiskEntries
  *
- * Old OSF/1 system call...
- * OSF/1 does this with a system call and library routine
- *
+ * Old OSF/1 system call.
+ * OSF/1 does this with a system call and library routine.
  *                  [mogul@wrl.dec.com (Jeffrey Mogul)]
+ *
  */
 
 # if defined (INITMOUNTNAMES)
@@ -915,18 +1115,18 @@ di_getDiskEntries (diskInfo, diCount)
     count = getmntinfo (&mntbufp, MNT_WAIT);
     if (count < 1)
     {
-        fprintf (stderr, "Unable to do getmntinfo () errno %d\n", errno);
+        fprintf (stderr, "Unable to do getmntinfo() errno %d\n", errno);
         return -1;
     }
 
     *diCount = count;
-    *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * count);
+    *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * (Size_t) count);
     if (*diskInfo == (diDiskInfo_t *) NULL)
     {
         fprintf (stderr, "malloc failed for diskInfo. errno %d\n", errno);
         return -1;
     }
-    memset ((char *) *diskInfo, '\0', sizeof (diDiskInfo_t) * count);
+    memset ((char *) *diskInfo, '\0', sizeof (diDiskInfo_t) * (Size_t) count);
 
     if (di_lib_debug > 1)
     {
@@ -953,12 +1153,12 @@ di_getDiskEntries (diskInfo, diCount)
 
         diptr->blockSize = (_fs_size_t) 1024;
 
-#  if _mem_f_fsize_statfs /* 1.x */
-        diptr->blockSize = (_fs_size_t) mntbufp [idx].f_fsize;
-#  endif
-#  if _mem_f_bsize_statfs /* 2.x */
-        diptr->blockSize = (_fs_size_t) mntbufp [idx].f_bsize;
-#  endif
+# if _mem_f_fsize_statfs /* OSF 1.x */
+       diptr->blockSize = (_fs_size_t) mntbufp [idx].f_fsize;
+# endif
+# if _mem_f_bsize_statfs /* OSF 2.x */
+       diptr->blockSize = (_fs_size_t) mntbufp [idx].f_bsize;
+# endif
         diptr->totalBlocks = (_fs_size_t) mntbufp [idx].f_blocks;
         diptr->freeBlocks = (_fs_size_t) mntbufp [idx].f_bfree;
         diptr->availBlocks = (_fs_size_t) mntbufp [idx].f_bavail;
@@ -981,7 +1181,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_UFS)
                 case MOUNT_UFS:         /* UNIX "Fast" Filesystem */
                 {
@@ -989,7 +1188,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_NFS)
                 case MOUNT_NFS:         /* Network Filesystem */
                 {
@@ -997,7 +1195,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_MFS)
                 case MOUNT_MFS:         /* Memory Filesystem */
                 {
@@ -1005,7 +1202,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_MSDOS)
                 case MOUNT_MSDOS:       /* MSDOS Filesystem */
                 {
@@ -1013,7 +1209,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_LFS)
                 case MOUNT_LFS:
                 {
@@ -1021,7 +1216,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_LOFS)
                 case MOUNT_LOFS:
                 {
@@ -1029,7 +1223,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_FDESC)
                 case MOUNT_FDESC:
                 {
@@ -1037,7 +1230,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_PORTAL)
                 case MOUNT_PORTAL:
                 {
@@ -1045,7 +1237,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_NULL)
                 case MOUNT_NULL:
                 {
@@ -1053,7 +1244,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_UMAP)
                 case MOUNT_UMAP:
                 {
@@ -1061,7 +1251,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_KERNFS)
                 case MOUNT_KERNFS:
                 {
@@ -1069,7 +1258,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_PROCFS)
                 case MOUNT_PROCFS:      /* proc filesystem */
                 {
@@ -1077,7 +1265,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_AFS)
                 case MOUNT_AFS:
                 {
@@ -1085,7 +1272,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_ISOFS)
                 case MOUNT_ISOFS:       /* iso9660 cdrom */
                 {
@@ -1093,7 +1279,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_ISO9660) && ! defined (MOUNT_CD9660)
                 case MOUNT_ISO9660:       /* iso9660 cdrom */
                 {
@@ -1101,7 +1286,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_CD9660)
                 case MOUNT_CD9660:       /* iso9660 cdrom */
                 {
@@ -1109,7 +1293,6 @@ di_getDiskEntries (diskInfo, diCount)
                     break;
                 }
 #  endif
-
 #  if defined (MOUNT_UNION)
                 case MOUNT_UNION:
                 {
@@ -1130,7 +1313,7 @@ di_getDiskEntries (diskInfo, diCount)
         }
         else
         {
-            Snprintf (SPF(diptr->fsType, sizeof (diptr->fsType),
+            Snprintf (diptr->fsType, DI_SPF(sizeof (diptr->fsType),
                       DI_UNKNOWN_FSTYPE), fstype);
         }
 #  endif
@@ -1195,7 +1378,7 @@ di_getDiskEntries (diskInfo, diCount)
     diDiskInfo_t     *diptr;
     int             count;
     int             idx;
-    long            bufsize;
+    Size_t          bufsize;
     struct statfs   *mntbufp;
     struct statfs   *sp;
 
@@ -1206,19 +1389,20 @@ di_getDiskEntries (diskInfo, diCount)
         fprintf (stderr, "Unable to do getfsstat () errno %d\n", errno);
         return -1;
     }
-    bufsize = sizeof (struct statfs) * count;
+    bufsize = sizeof (struct statfs) * (Size_t) count;
     mntbufp = malloc (bufsize);
-    memset ((char *) mntbufp, '\0', sizeof (struct statfs) * count);
-    count = getfsstat (mntbufp, bufsize, MNT_NOWAIT);
+    memset ((char *) mntbufp, '\0', sizeof (struct statfs) * (Size_t) count);
+    count = getfsstat (mntbufp, (long) bufsize, MNT_NOWAIT);
 
     *diCount = count;
-    *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * count);
+    *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * (Size_t) count);
+
     if (*diskInfo == (diDiskInfo_t *) NULL)
     {
         fprintf (stderr, "malloc failed for diskInfo. errno %d\n", errno);
         return -1;
     }
-    memset ((char *) *diskInfo, '\0', sizeof (diDiskInfo_t) * count);
+    memset ((char *) *diskInfo, '\0', sizeof (diDiskInfo_t) * (Size_t) count);
 
     for (idx = 0; idx < count; idx++)
     {
@@ -1337,11 +1521,11 @@ di_getDiskEntries (diskInfo, diCount)
     }
     bufsize = sizeof (struct statvfs) * count;
     mntbufp = malloc (bufsize);
-    memset ((char *) mntbufp, '\0', sizeof (struct statvfs) * count);
+    memset ((char *) mntbufp, '\0', sizeof (struct statvfs) * (Size_t) count);
     count = getvfsstat (mntbufp, bufsize, ST_NOWAIT);
 
     *diCount = count;
-    *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * count);
+    *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * (Size_t) count);
     if (*diskInfo == (diDiskInfo_t *) NULL)
     {
         fprintf (stderr, "malloc failed for diskInfo. errno %d\n", errno);
@@ -1442,7 +1626,7 @@ di_getDiskEntries (diskInfo, diCount)
 {
     diDiskInfo_t     *diptr;
     int             count;
-    int             bufsize;
+    Size_t          bufsize;
     int             idx;
     short           fstype;
     struct fs_data  *fsdbuf;
@@ -1471,7 +1655,7 @@ di_getDiskEntries (diskInfo, diCount)
     }
 
     *diCount = count;
-    *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * count);
+    *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * (Size_t) count);
     if (*diskInfo == (diDiskInfo_t *) NULL)
     {
         fprintf (stderr, "malloc failed for diskInfo. errno %d\n", errno);
@@ -1520,7 +1704,7 @@ di_getDiskEntries (diskInfo, diCount)
         }
         else
         {
-            Snprintf (SPF(diptr->fsType, sizeof (diptr->fsType),
+            Snprintf (diptr->fsType, DI_SPF(sizeof (diptr->fsType),
                           "Unknown fstyp %.2d"), fstype);
         }
 
@@ -1594,7 +1778,7 @@ di_getDiskEntries (diskInfo, diCount)
     diDiskInfo_t     *diptr;
     int             num;        /* number of vmount structs returned    */
     char            *vmbuf;     /* buffer for vmount structs returned   */
-    int             vmbufsz;    /* size in bytes of vmbuf               */
+    Size_t          vmbufsz;    /* size in bytes of vmbuf               */
     int             i;          /* index for looping and stuff          */
     char            *bufp;      /* pointer into vmbuf                   */
     struct vmount   *vmtp;      /* pointer into vmbuf                   */
@@ -1724,8 +1908,7 @@ di_getDiskEntries (diskInfo, diCount)
 
 #endif  /* _lib_mntctl */
 
-#if (_statfs_2arg || \
-        _statfs_3arg) && \
+#if _lib_statfs && (_statfs_2arg ||_statfs_3arg) && \
         ! defined (_lib_statvfs) && \
         ! defined (_lib_getmntinfo) && \
         ! defined (_lib_getmnt) && \
@@ -1758,7 +1941,8 @@ di_getDiskInfo (diskInfo, diCount)
     for (i = 0; i < *diCount; ++i)
     {
         diptr = *diskInfo + i;
-        if (diptr->printFlag == DI_PRNT_OK)
+        if (diptr->printFlag == DI_PRNT_OK ||
+            diptr->printFlag == DI_PRNT_FORCE)
         {
             if (statfs (diptr->name, &statBuf) == 0)
             {
@@ -1913,7 +2097,8 @@ di_getDiskInfo (diskInfo, diCount)
     for (i = 0; i < *diCount; ++i)
     {
         diptr = *diskInfo + i;
-        if (diptr->printFlag == DI_PRNT_OK)
+        if (diptr->printFlag == DI_PRNT_OK ||
+            diptr->printFlag == DI_PRNT_FORCE)
         {
             rc = GetVolumeInformation (diptr->name,
                     volName, MSDOS_BUFFER_SIZE, &serialNo, &maxCompLen,
@@ -2209,6 +2394,56 @@ convertMountOptions (flags, diptr)
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
+#if defined (MNT_REMOVABLE)
+    if ((flags & MNT_REMOVABLE) == MNT_REMOVABLE)
+    {
+        strncat (diptr->options, "removable,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_PERSISTENT)
+    if ((flags & MNT_PERSISTENT) == MNT_PERSISTENT)
+    {
+        strncat (diptr->options, "persistent,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_SHARED)
+    if ((flags & MNT_SHARED) == MNT_SHARED)
+    {
+        strncat (diptr->options, "shared,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_BLOCKBASED)
+    if ((flags & MNT_BLOCKBASED) == MNT_BLOCKBASED)
+    {
+        strncat (diptr->options, "blockbased,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_HAS_MIME)
+    if ((flags & MNT_HAS_MIME) == MNT_HAS_MIME)
+    {
+        strncat (diptr->options, "mime,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_HAS_QUERY)
+    if ((flags & MNT_HAS_QUERY) == MNT_HAS_QUERY)
+    {
+        strncat (diptr->options, "query,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_HAS_ATTR)
+    if ((flags & MNT_HAS_ATTR) == MNT_HAS_ATTR)
+    {
+        strncat (diptr->options, "attr,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+    return;
 }
 
 static void
@@ -2234,7 +2469,7 @@ convertNFSMountOptions (flags, wsize, rsize, diptr)
     {
         char          tmp [64];
 
-        Snprintf (SPF(tmp, sizeof (tmp), "wsize=%ld,"), wsize);
+        Snprintf (tmp, DI_SPF(sizeof (tmp), "wsize=%ld,"), wsize);
         strncat (diptr->options, tmp,
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
@@ -2244,7 +2479,7 @@ convertNFSMountOptions (flags, wsize, rsize, diptr)
     {
         char          tmp [64];
 
-        Snprintf (SPF(tmp, sizeof (tmp), "rsize=%ld,"), rsize);
+        Snprintf (tmp, DI_SPF(sizeof (tmp), "rsize=%ld,"), rsize);
         strncat (diptr->options, tmp,
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
@@ -2264,6 +2499,7 @@ convertNFSMountOptions (flags, wsize, rsize, diptr)
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
+    return;
 }
 
 
@@ -2276,7 +2512,7 @@ trimChar (str, ch)
     int          ch;
 #endif
 {
-    int               len;
+    unsigned int  len;
 
     len = strlen (str);
     if (len > 0)
