@@ -1,8 +1,8 @@
-#!/usr/local/bin/perl
+#!/usr/bin/perl
 #
-# $Id: mkconfig.pl,v 1.26 2008-05-25 20:32:54-07 bll Exp $
+# $Id: mkconfig.pl,v 1.31 2009-11-22 09:16:37-08 bll Exp $
 #
-# Copyright 2006-2008 Brad Lanam Walnut Creek, CA USA
+# Copyright 2006-2009 Brad Lanam Walnut Creek, CA USA
 #
 
 # HP-UX doesn't have these installed.
@@ -71,62 +71,16 @@ check_link
     my ($name, $code, $r_clist, $r_config, $r_a) = @_;
 
     my $otherlibs = '';
-    if (defined ($$r_a{'otherlibs'}))
+    if (defined ($r_a->{'otherlibs'}))
     {
-        $otherlibs = $$r_a{'otherlibs'};
+        $otherlibs = $r_a->{'otherlibs'};
     }
 
     open (CLFH, ">$name.c");
-
     print CLFH $precc;
 
-    if ($$r_a{'incheaders'} eq 'all' ||
-        $$r_a{'incheaders'} eq 'std')
-    {
-        # always include these four if present ...
-        foreach my $val ('_hdr_stdio', '_hdr_stdlib', '_sys_types', '_sys_param')
-        {
-            if (defined ($$r_config{$val}) &&
-                 $$r_config{$val} ne '0')
-            {
-                print CLFH "#include <$$r_config{$val}>\n";
-            }
-        }
-    }
-
-    if ($$r_a{'incheaders'} eq 'all')
-    {
-        foreach my $val (@$r_clist)
-        {
-            if ($val !~ m#^(_hdr_|_sys_)#o)
-            {
-                next;
-            }
-            if ($val eq '_hdr_stdio' ||
-                $val eq '_hdr_stdlib' ||
-                $val eq '_sys_types' ||
-                $val eq '_sys_param')
-            {
-                next;
-            }
-            if ($val eq '_hdr_malloc' &&
-                $$r_config{'_include_malloc'} eq '0')
-            {
-                next;
-            }
-            if ($val eq '_hdr_strings' &&
-                $$r_config{'_hdr_string'} ne '0' &&
-                $$r_config{'_include_string'} eq '0')
-            {
-                next;
-            }
-            if ($$r_config{$val} ne '0')
-            {
-                print CLFH "#include <$$r_config{$val}>\n";
-            }
-        }
-        print CLFH "\n";
-    }
+    my $hdrs = print_headers ($r_a, $r_clist, $r_config);
+    print CLFH $hdrs;
     print CLFH $code;
     close CLFH;
 
@@ -137,46 +91,52 @@ check_link
     my $rc = _check_link ($name, {} );
     if ($rc != 0)
     {
-        if ($otherlibs ne '')
+      if ($otherlibs ne '')
+      {
+        my @olibs = split (/\s+/, $otherlibs);
+        $oliblist = '';
+        foreach my $olib (@olibs)
         {
-            $rc = _check_link ($name, { 'otherlibs' => $otherlibs, } );
-            if ($rc == 0)
-            {
-                my $r_hash = $$r_config{'reqlibs'};
-                my @vals = split (/\s+/, $otherlibs);
-                foreach my $val (@vals)
-                {
-                    if ($val eq '') { next; }
-                    $$r_hash{$val} = 1;
-                    $dlibs .= $val . ' ';
-                }
-            }
+          $oliblist = $oliblist . ' ' . $olib;
+          $rc = _check_link ($name, { 'otherlibs' => $oliblist, } );
+          if ($rc == 0)
+          {
+              my $r_hash = $r_config->{'reqlibs'};
+              my @vals = split (/\s+/, $oliblist);
+              $dlibs = '';
+              foreach my $val (@vals)
+              {
+                  if ($val eq '') { next; }
+                  $r_hash->{$val} = 1;
+                  $dlibs .= $val . ' ';
+              }
+              last;
+          }
         }
-        if ($rc != 0 && $$r_a{'tryextern'} == 1)
+      }
+
+      if ($rc != 0 && $r_a->{'tryextern'} == 1)
+      {
+        $rc = _check_link ($name, { 'cflags' => '-D_TRY_extern_=1',
+            'otherlibs' => $otherlibs, } );
+        if ($rc == 0)
         {
-            $rc = _check_link ($name, { 'cflags' => '-D_TRY_extern_', } );
-            if ($rc != 0 && $otherlibs ne '')
-            {
-                $rc = _check_link ($name, { 'cflags' => '-D_TRY_extern_',
-                    'otherlibs' => $otherlibs, } );
-                if ($rc == 0)
-                {
-                    my $r_hash = $$r_config{'reqlibs'};
-                    my @vals = split (/\s+/, $otherlibs);
-                    foreach my $val (@vals)
-                    {
-                        if ($val eq '') { next; }
-                        $$r_hash{$val} = 1;
-                        $dlibs .= $val . ' ';
-                    }
-                }
-            }
+          my $r_hash = $r_config->{'reqlibs'};
+          my @vals = split (/\s+/, $otherlibs);
+          $dlibs = '';
+          foreach my $val (@vals)
+          {
+              if ($val eq '') { next; }
+              $r_hash->{$val} = 1;
+              $dlibs .= $val . ' ';
+          }
         }
+      }
     }
 
-    $$r_a{'dlibs'} = $dlibs;
+    $r_a->{'dlibs'} = $dlibs;
 
-    if ($$r_a{'nounlink'} == 0)
+    if ($r_a->{'nounlink'} == 0)
     {
         unlink "$name.exe";
         unlink "$name.c";
@@ -191,15 +151,15 @@ _check_link
     my ($name, $r_a) = @_;
 
     my $cmd = "$ENV{'CC'} $ENV{'CFLAGS'} ";
-    if (defined ($$r_a{'cflags'}))
+    if (defined ($r_a->{'cflags'}))
     {
-        $cmd .= ' ' . $$r_a{'cflags'} . ' ';
+        $cmd .= ' ' . $r_a->{'cflags'} . ' ';
     }
     $cmd .= "-o $name.exe $name.c";
     $cmd .= " $ENV{'LDFLAGS'} $ENV{'LIBS'}";
-    if (defined ($$r_a{'otherlibs'}) && $$r_a{'otherlibs'} ne undef)
+    if (defined ($r_a->{'otherlibs'}) && $r_a->{'otherlibs'} ne undef)
     {
-        $cmd .= ' ' . $$r_a{'otherlibs'} . ' ';
+        $cmd .= ' ' . $r_a->{'otherlibs'} . ' ';
     }
     print LOGFH "##  link test: $cmd\n";
     my $rc = system ("$cmd >> $LOG 2>&1");
@@ -224,53 +184,8 @@ check_compile
 
     print CCFH $precc;
 
-    if ($$r_a{'incheaders'} eq 'std' ||
-        $$r_a{'incheaders'} eq 'all')
-    {
-        # always include these four if present ...
-        foreach my $val ('_hdr_stdio', '_hdr_stdlib', '_sys_types', '_sys_param')
-        {
-            if (defined ($$r_config{$val}) &&
-                 $$r_config{$val} ne '0')
-            {
-                print CCFH "#include <$$r_config{$val}>\n";
-            }
-        }
-    }
-
-    if ($$r_a{'incheaders'} eq 'all')
-    {
-        foreach my $val (@$r_clist)
-        {
-            if ($val !~ m#^(_hdr_|_sys_)#o)
-            {
-                next;
-            }
-            if ($val eq '_hdr_stdio' ||
-                $val eq '_hdr_stdlib' ||
-                $val eq '_sys_types' ||
-                $val eq '_sys_param')
-            {
-                next;
-            }
-            if ($val eq '_hdr_malloc' &&
-                $$r_config{'_include_malloc'} eq '0')
-            {
-                next;
-            }
-            if ($val eq '_hdr_strings' &&
-                $$r_config{'_hdr_string'} ne '0' &&
-                $$r_config{'_include_string'} eq '0')
-            {
-                next;
-            }
-            if ($$r_config{$val} ne '0')
-            {
-                print CCFH "#include <$$r_config{$val}>\n";
-            }
-        }
-        print CCFH "\n";
-    }
+    my $hdrs = print_headers ($r_a, $r_clist, $r_config);
+    print CCFH $hdrs;
     print CCFH $code;
     close CCFH;
 
@@ -294,54 +209,8 @@ check_cpp
     open (CCFH, ">$name.c");
 
     print CCFH $precc;
-
-    if ($$r_a{'incheaders'} eq 'std' ||
-        $$r_a{'incheaders'} eq 'all')
-    {
-        # always include these four if present ...
-        foreach my $val ('_hdr_stdio', '_hdr_stdlib', '_sys_types', '_sys_param')
-        {
-            if (defined ($$r_config{$val}) &&
-                 $$r_config{$val} ne '0')
-            {
-                print CCFH "#include <$$r_config{$val}>\n";
-            }
-        }
-    }
-
-    if ($$r_a{'incheaders'} eq 'all')
-    {
-        foreach my $val (@$r_clist)
-        {
-            if ($val !~ m#^(_hdr_|_sys_)#o)
-            {
-                next;
-            }
-            if ($val eq '_hdr_stdio' ||
-                $val eq '_hdr_stdlib' ||
-                $val eq '_sys_types' ||
-                $val eq '_sys_param')
-            {
-                next;
-            }
-            if ($val eq '_hdr_malloc' &&
-                $$r_config{'_include_malloc'} eq '0')
-            {
-                next;
-            }
-            if ($val eq '_hdr_strings' &&
-                $$r_config{'_hdr_string'} ne '0' &&
-                $$r_config{'_include_string'} eq '0')
-            {
-                next;
-            }
-            if ($$r_config{$val} ne '0')
-            {
-                print CCFH "#include <$$r_config{$val}>\n";
-            }
-        }
-        print CCFH "\n";
-    }
+    my $hdrs = print_headers ($r_a, $r_clist, $r_config);
+    print CCFH $hdrs;
     print CCFH $code;
     close CCFH;
 
@@ -361,8 +230,8 @@ check_dashe
 {
     my ($name, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] supports: cc -E ... \n";
-    print STDERR "supports: cc -E ... ";
+    print LOGFH "## [$name] $ENV{'CC'} supports -E ... \n";
+    print STDERR "$ENV{'CC'} supports -E ... ";
     my $code = <<"_HERE_";
 main () { exit (0); }
 _HERE_
@@ -379,22 +248,30 @@ _HERE_
         print LOGFH "## [$name] no\n";
         print STDERR "no\n";
     }
-    $$r_config{$name} = $rc;
+    $r_config->{$name} = $rc;
 }
 
 sub
 check_header
 {
-    my ($name, $file, $r_clist, $r_config) = @_;
+    my ($name, $file, $r_clist, $r_config, $r_a) = @_;
 
     print LOGFH "## [$name] header: $file ... \n";
     print STDERR "header: $file ... ";
-    my $code = <<"_HERE_";
+    my $r_rh = $r_a->{'reqhdr'} || [];
+    my $code = '';
+    foreach my $reqhdr (@$r_rh)
+    {
+        $code .= <<"_HERE_";
+#include <$reqhdr>
+_HERE_
+    }
+    $code .= <<"_HERE_";
 #include <${file}>
 main () { exit (0); }
 _HERE_
     my $rc = 1;
-    if ($$r_config{'_supports_dash_e'} eq '1')
+    if ($r_config->{'_supports_dash_e'} eq '1')
     {
         $rc = check_cpp ($name, $code, $r_clist, $r_config,
             { 'incheaders' => 'std', });
@@ -417,26 +294,136 @@ _HERE_
         print STDERR "no\n";
     }
     push @$r_clist, $name;
-    $$r_config{$name} = $val;
+    $r_config->{$name} = $val;
 }
 
 sub
-check_void
+print_headers
 {
-    my ($name, $r_clist, $r_config) = @_;
+    my ($r_a, $r_clist, $r_config) = @_;
+    my $txt;
 
-    print LOGFH "## [$name] supported: void ... \n";
-    print STDERR "supported: void ... ";
+    $txt = '';
+
+    if ($r_a->{'incheaders'} eq 'all' ||
+        $r_a->{'incheaders'} eq 'std')
+    {
+        # always include these four if present ...
+        foreach my $val ('_hdr_stdio', '_hdr_stdlib', '_sys_types', '_sys_param')
+        {
+            if (defined ($r_config->{$val}) &&
+                 $r_config->{$val} ne '0')
+            {
+                $txt .= "#include <" . $r_config->{$val} . ">\n";
+            }
+        }
+    }
+
+    if ($r_a->{'incheaders'} eq 'all')
+    {
+        foreach my $val (@$r_clist)
+        {
+            if ($val !~ m#^(_hdr_|_sys_)#o)
+            {
+                next;
+            }
+            if ($val eq '_hdr_stdio' ||
+                $val eq '_hdr_stdlib' ||
+                $val eq '_sys_types' ||
+                $val eq '_sys_param')
+            {
+                next;
+            }
+            if ($val eq '_hdr_malloc' &&
+                $r_config->{'_include_malloc'} eq '0')
+            {
+                next;
+            }
+            if ($val eq '_hdr_strings' &&
+                $r_config->{'_hdr_string'} ne '0' &&
+                $r_config->{'_include_string'} eq '0')
+            {
+                next;
+            }
+            if ($r_config->{$val} ne '0')
+            {
+                $txt .= "#include <" . $r_config->{$val} . ">\n";
+            }
+        }
+        $txt .= "\n";
+    }
+
+    return $txt;
+}
+
+sub
+check_class
+{
+    my ($name, $class, $r_clist, $r_config, $r_a) = @_;
+
+    push @$r_clist, $name;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
-main () { void *x; x = (char *) NULL; exit (0); }
+main () { $class testclass; }
+_HERE_
+    my $val = $r_a->{'otherlibs'} || '';
+
+    if ($val ne '')
+    {
+        print LOGFH "## [$name] class: $class [$val] ... \n";
+        print STDERR "class: $class [$val] ... ";
+    }
+    else
+    {
+        print LOGFH "## [$name] class: $class ... \n";
+        print STDERR "class: $class ... ";
+    }
+
+    my %a = (
+         'incheaders' => 'all',
+         'nounlink' => 0,
+         'otherlibs' => $val,
+         'tryextern' => 0,
+         );
+    my $rc = check_link ($name, $code, $r_clist, $r_config, \%a);
+    if ($rc == 0)
+    {
+      $r_config->{$name} = 1;
+      print LOGFH "## [$name] yes";
+      print STDERR "yes";
+      if ($a{'dlibs'} ne '')
+      {
+          print LOGFH " with $a{'dlibs'}";
+          print STDERR " with $a{'dlibs'}";
+      }
+      print LOGFH "\n";
+      print STDERR "\n";
+    }
+    else
+    {
+      print LOGFH "## [$name] no\n";
+      print STDERR "no\n";
+    }
+}
+
+# if the keyword is reserved, the compile will fail.
+sub
+check_keyword
+{
+    my ($name, $keyword, $r_clist, $r_config) = @_;
+
+    print LOGFH "## [$name] keyword: $keyword ... \n";
+    print STDERR "keyword: $keyword ... ";
+    $r_config->{$name} = 0;
+    my $code = <<"_HERE_";
+main () { int ${keyword}; ${keyword} = 1; exit (0); }
 _HERE_
     my $rc = check_compile ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'std', });
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
-    if ($rc == 0)
+    if ($rc != 0)  # failure means it is reserved...
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -463,10 +450,10 @@ _HERE_
     my $rc = check_compile ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'all', });
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -486,17 +473,17 @@ check_command
     print STDERR "command: $cmd ... ";
 
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     foreach my $p (split /[;:]/o, $ENV{'PATH'})
     {
-        if (-x "$p/$cmd" && $$r_config{$name} == 0)
+        if (-x "$p/$cmd" && $r_config->{$name} == 0)
         {
-            $$r_config{$name} = 1;
+            $r_config->{$name} = 1;
             print LOGFH "## [$name] yes\n";
             print STDERR "yes\n";
         }
     }
-    if ($$r_config{$name} == 0)
+    if ($r_config->{$name} == 0)
     {
         print LOGFH "## [$name] no\n";
         print STDERR "no\n";
@@ -510,8 +497,8 @@ check_include_malloc
     my ($name, $r_clist, $r_config) = @_;
 
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
-    if ($$r_config{'_hdr_malloc'} ne '0')
+    $r_config->{$name} = 0;
+    if ($r_config->{'_hdr_malloc'} ne '0')
     {
         print LOGFH "## [$name] header: include malloc.h ... \n";
         print STDERR "header: include malloc.h ... ";
@@ -527,7 +514,7 @@ _HERE_
                 { 'incheaders' => 'all', });
         if ($rc == 0)
         {
-            $$r_config{$name} = 1;
+            $r_config->{$name} = 1;
             print LOGFH "## [$name] yes\n";
             print STDERR "yes\n";
         }
@@ -545,9 +532,9 @@ check_include_string
     my ($name, $r_clist, $r_config) = @_;
 
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
-    if ($$r_config{'_hdr_string'} ne '0' &&
-        $$r_config{'_hdr_strings'} ne '0')
+    $r_config->{$name} = 0;
+    if ($r_config->{'_hdr_string'} ne '0' &&
+        $r_config->{'_hdr_strings'} ne '0')
     {
         print LOGFH "## [$name] header: include both string.h & strings.h ... \n";
         print STDERR "header: include both string.h & strings.h ... ";
@@ -564,7 +551,7 @@ _HERE_
                 { 'incheaders' => 'std', });
         if ($rc == 0)
         {
-            $$r_config{$name} = 1;
+            $r_config->{$name} = 1;
             print LOGFH "## [$name] yes\n";
             print STDERR "yes\n";
         }
@@ -584,7 +571,7 @@ check_npt
     print LOGFH "## [$name] need prototype: $proto ... \n";
     print STDERR "need prototype: $proto ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 _BEGIN_EXTERNS_
 struct _TEST_struct { int _TEST_member; };
@@ -595,7 +582,7 @@ _HERE_
             { 'incheaders' => 'all', });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -611,10 +598,10 @@ check_type
 {
     my ($name, $type, $r_clist, $r_config) = @_;
 
-    print LOGFH "## [$name] type $type ... \n";
-    print STDERR "type $type ... ";
+    print LOGFH "## [$name] type: $type ... \n";
+    print STDERR "type: $type ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 struct xxx { $type mem; };
 static struct xxx v;
@@ -625,7 +612,7 @@ _HERE_
             { 'incheaders' => 'all', });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -642,7 +629,7 @@ check_lib
     my ($name, $func, $r_clist, $r_config, $r_a) = @_;
 
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 typedef int (*_TEST_fun_)();
 #ifdef _TRY_extern_
@@ -653,7 +640,7 @@ _END_EXTERNS_
 static _TEST_fun_ i=(_TEST_fun_) $func;
 main () {  return (i==0); }
 _HERE_
-    my $val = $$r_a{'otherlibs'} || '';
+    my $val = $r_a->{'otherlibs'} || '';
 
     if ($val ne '')
     {
@@ -670,26 +657,26 @@ _HERE_
          'incheaders' => 'all',
          'nounlink' => 0,
          'otherlibs' => $val,
-         'tryextern' => 1,
+         'tryextern' => 0,
          );
     my $rc = check_link ($name, $code, $r_clist, $r_config, \%a);
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
-        print LOGFH "## [$name] yes";
-        print STDERR "yes";
-        if ($a{'dlibs'} ne '')
-        {
-            print LOGFH " with $a{'dlibs'}";
-            print STDERR " with $a{'dlibs'}";
-        }
-        print LOGFH "\n";
-        print STDERR "\n";
+      $r_config->{$name} = 1;
+      print LOGFH "## [$name] yes";
+      print STDERR "yes";
+      if ($a{'dlibs'} ne '')
+      {
+          print LOGFH " with $a{'dlibs'}";
+          print STDERR " with $a{'dlibs'}";
+      }
+      print LOGFH "\n";
+      print STDERR "\n";
     }
     else
     {
-        print LOGFH "## [$name] no\n";
-        print STDERR "no\n";
+      print LOGFH "## [$name] no\n";
+      print STDERR "no\n";
     }
 }
 
@@ -701,7 +688,7 @@ check_setmntent_1arg
     print LOGFH "## [$name] setmntent(): 1 argument ... \n";
     print STDERR "setmntent(): 1 argument ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () { setmntent ("/etc/mnttab"); }
 _HERE_
@@ -709,7 +696,7 @@ _HERE_
         { 'incheaders' => 'all', 'nounlink' => 0, 'otherlibs' => undef, 'tryextern' => 0, });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -728,7 +715,7 @@ check_setmntent_2arg
     print LOGFH "## [$name] setmntent(): 2 arguments ... \n";
     print STDERR "setmntent(): 2 arguments ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () { setmntent ("/etc/mnttab", "r"); }
 _HERE_
@@ -736,7 +723,7 @@ _HERE_
         { 'incheaders' => 'all', 'nounlink' => 0, 'otherlibs' => undef, 'tryextern' => 0, });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -755,7 +742,7 @@ check_statfs_2arg
     print LOGFH "## [$name] statfs(): 2 arguments ... \n";
     print STDERR "statfs(): 2 arguments ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () {
     struct statfs statBuf; char *name; name = "/";
@@ -766,7 +753,7 @@ _HERE_
         { 'incheaders' => 'all', 'nounlink' => 0, 'otherlibs' => undef, 'tryextern' => 0, });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -785,7 +772,7 @@ check_statfs_3arg
     print LOGFH "## [$name] statfs(): 3 arguments ... \n";
     print STDERR "statfs(): 3 arguments ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () {
     struct statfs statBuf; char *name; name = "/";
@@ -796,7 +783,7 @@ _HERE_
         { 'incheaders' => 'all', 'nounlink' => 0, 'otherlibs' => undef, 'tryextern' => 0, });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -815,7 +802,7 @@ check_statfs_4arg
     print LOGFH "## [$name] statfs(): 4 arguments ... \n";
     print STDERR "statfs(): 4 arguments ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () {
     struct statfs statBuf; char *name; name = "/";
@@ -826,7 +813,7 @@ _HERE_
         { 'incheaders' => 'all', 'nounlink' => 0, 'otherlibs' => undef, 'tryextern' => 0, });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -845,7 +832,7 @@ check_size
     print LOGFH "## [$name] sizeof: $type ... \n";
     print STDERR "sizeof: $type ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () {
 	printf("%u\\n", sizeof($type));
@@ -856,7 +843,7 @@ _HERE_
     my $rc = check_run ($name, $code, \$val, $r_clist, $r_config, {});
     if ($rc == 0)
     {
-        $$r_config{$name} = $val;
+        $r_config->{$name} = $val;
         print LOGFH "## [$name] $val\n";
         print STDERR "$val\n";
     }
@@ -875,7 +862,7 @@ check_member
     print LOGFH "## [$name] exists: $struct.$member ... \n";
     print STDERR "exists: $struct.$member ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () { struct $struct s; int i; i = sizeof (s.$member); }
 _HERE_
@@ -883,7 +870,7 @@ _HERE_
             { 'incheaders' => 'all', });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -902,7 +889,7 @@ check_int_declare
     print LOGFH "## [$name] declared: $function ... \n";
     print STDERR "declared: $function ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
     main () { int x; x = $function; }
 _HERE_
@@ -910,7 +897,7 @@ _HERE_
             { 'incheaders' => 'all', });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -929,7 +916,7 @@ check_ptr_declare
     print LOGFH "## [$name] declared: $function ... \n";
     print STDERR "declared: $function ... ";
     push @$r_clist, $name;
-    $$r_config{$name} = 0;
+    $r_config->{$name} = 0;
     my $code = <<"_HERE_";
 main () { _VOID_ *x; x = $function; }
 _HERE_
@@ -937,7 +924,7 @@ _HERE_
             { 'incheaders' => 'all', });
     if ($rc == 0)
     {
-        $$r_config{$name} = 1;
+        $r_config->{$name} = 1;
         print LOGFH "## [$name] yes\n";
         print STDERR "yes\n";
     }
@@ -977,9 +964,11 @@ _HERE_
 
     foreach my $r_arr (@headlist1)
     {
-        check_header ($$r_arr[0], $$r_arr[1], \@clist, \%config);
+        check_header ($$r_arr[0], $$r_arr[1], \@clist, \%config,
+                { 'reqhdr' => [], });
     }
-    check_void ('_key_void', \@clist, \%config);
+    check_keyword ('_key_void', 'void', \@clist, \%config);
+    check_keyword ('_key_const', 'const', \@clist, \%config);
     check_proto ('_proto_stdc', \@clist, \%config);
 
     if (! open (DATAIN, '../' . $ARGV[0]))
@@ -1022,24 +1011,30 @@ _HERE_
             print LOGFH "start include\n";
             $ininclude = 1;
         }
-        elsif ($line =~ m#^(hdr|sys)\s+(.*)#o)
+        elsif ($line =~ m#^(hdr|sys)\s+([^\s]+)\s*(.*)#o)
         {
             my $typ = $1;
             my $hdr = $2;
+            my $reqhdr = $3;
             my $nm = "_${typ}_";
-            # all this rigamarole is to match what iffe does.
+            # create the standard header name for config.h
             my @h = split (/\//, $hdr);
             $h[0] = lc $h[0];
             $nm .= join ('_', @h);
             $nm =~ s,\.h$,,o;
+            $nm =~ s,:,_,go;
             if ($typ eq 'sys')
             {
                 $hdr = 'sys/' . $hdr;
             }
+            $reqhdr =~ s/^\s*//o;
+            $reqhdr =~ s/\s*$//o;
+            my @oh = split (/\s+/, $reqhdr);
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
-                check_header ($nm, $hdr, \@clist, \%config);
+                check_header ($nm, $hdr, \@clist, \%config,
+                    { 'reqhdr' => \@oh, });
 
                 if ((($hdr eq 'strings.h' &&
                       defined ($config{'_hdr_string'}) &&
@@ -1078,6 +1073,29 @@ _HERE_
                 }
             }
         }
+        elsif ($line =~ m#^key\s+(.*)#o)
+        {
+            my $tnm = $1;
+            my $nm = "_key_" . lc $tnm;
+            if (! defined ($config{$nm}) ||
+                $config{$nm} eq '0')
+            {
+                check_keyword ($nm, $tnm, \@clist, \%config);
+            }
+        }
+        elsif ($line =~ m#^class\s+([^\s]+)\s*(.*)?#o)
+        {
+            my $class = $1;
+            my $libs = $2 || '';
+            my $nm = "_class_" . $class;
+            $nm =~ s,:,_,go;
+            if (! defined ($config{$nm}) ||
+                $config{$nm} eq '0')
+            {
+                check_class ($nm, $class, \@clist, \%config,
+                       { 'otherlibs' => $libs, });
+            }
+        }
         elsif ($line =~ m#^typ\s+(.*)#o)
         {
             my $tnm = $1;
@@ -1184,6 +1202,9 @@ _HERE_
 #if ! _key_void || ! _proto_stdc
 # define void int
 #endif
+#if ! _key_const || ! _proto_stdc
+# define const
+#endif
 
 #ifndef _
 # if _proto_stdc
@@ -1230,7 +1251,7 @@ print LOGFH "CFLAGS: $ENV{'CFLAGS'}\n";
 print LOGFH "LDFLAGS: $ENV{'LDFLAGS'}\n";
 print LOGFH "LIBS: $ENV{'LIBS'}\n";
 
-&create_config;
+create_config;
 
 close LOGFH;
 
